<?php

namespace App\Controller;

use App\Entity\Snip;
use App\Security\Voter\SnipVoter;
use App\Service\SnipServiceFactory;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/history/{snip}', name: 'history')]
class HistoryController extends AbstractController
{
    public function __construct(
        private readonly SnipServiceFactory $snipServiceFactory,
    ) {}

    #[Route('/', name: '_index')]
    public function index(Snip $snip): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $snipService = $this->snipServiceFactory->create($snip);
        return $this->render('history/index.html.twig', [
            'snip' => $snip,
            'versions' => $snipService->getVersions(),
            'latestVersion' => $snipService->getLatestVersion(),
        ]);
    }

    #[Route('/set/{version}', name: '_set')]
    public function set(Snip $snip, string $version): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $this->snipServiceFactory->create($snip)->setVersion($version);
        $this->addFlash('success', 'Snip version set to ' . $version);
        return $this->redirectToRoute('snip_single', ['snip' => $snip->getId()]);
    }
}