<?php

namespace App\Service\SnipContent;

use App\Entity\User;
use App\Git\CustomGitRepository;
use App\Git\SimpleCommit;
use Symfony\Component\Security\Core\User\UserInterface;

class SnipContentGit implements SnipContentInterface
{
    private const string SNIP_FILE_NAME = 'snip.txt';
    private const string MASTER_BRANCH_NAME = 'master';

    public function __construct(
        private readonly CustomGitRepository $repo,
        private readonly ?User               $user,
    ) {}

    private function snipExists(): bool
    {
        return file_exists($this->getSnipPath());
    }

    private function getSnipPath(): string
    {
        return sprintf('%s/snip.txt', $this->repo->getRepositoryPath());
    }

    public function update(string $snipContents): void
    {
        if (!$this->user instanceof UserInterface) {
            return;
        }
        if ($this->repo->getCurrentBranchName() !== self::MASTER_BRANCH_NAME) {
            $this->repo->checkout(self::MASTER_BRANCH_NAME);
        }
        file_put_contents($this->getSnipPath(), $snipContents);
        $this->repo->addFile(self::SNIP_FILE_NAME);
        if ($this->repo->hasChanges()) {
            $this->repo->commit(sprintf('Updated snip at %s by %s', date('Y-m-d H:i:s'), $this->user));
        }
    }

    public function get(): string
    {
        if (!$this->snipExists()) {
            return '';
        }
        return file_get_contents($this->getSnipPath());
    }

    public function getVersions(): array
    {
        return array_map(fn(SimpleCommit $c) => [
            'id' => $c->getHash(),
            'name' => $c->getDate()->format('Y-m-d H:i:s'),
        ], $this->repo->getAllCommits());
    }

    public function setVersion(string $version): void
    {
        $this->repo->checkout($version);
    }

    public function getCommit(): string
    {
        return $this->repo->getCurrentBranchName();
    }

    public function delete(): void
    {
        system("rm -rf " . escapeshellarg($this->repo->getRepositoryPath()));
    }

    public function getLatestVersion(): string
    {
        return self::MASTER_BRANCH_NAME;
    }
}